﻿Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput


Namespace HorizontalAttributes

    Public Class Commands

        ' Class variable to store the instance of our overrule
        Private Shared myOverrule As KeepStraightOverrule

        <CommandMethod("KeepStraight")> _
        Public Shared Sub ImplementOverrule()

            Dim doc As Document = _
            Application.DocumentManager.MdiActiveDocument()
            Dim ed As Editor = doc.Editor

            ' Select a block reference

            Dim opts As New PromptEntityOptions( _
              vbLf & "Select a block reference:")
            opts.SetRejectMessage(vbLf & "Must be block reference...")
            opts.AddAllowedClass(GetType(BlockReference), True)

            Dim res As PromptEntityResult = ed.GetEntity(opts)

            If res.Status <> PromptStatus.OK Then
                Exit Sub
            End If

            Dim objIds() As ObjectId
            Dim db As Database = doc.Database
            Using trans As Transaction = _
                db.TransactionManager.StartTransaction()

                ' Open the BlockReference for read.
                ' We know its a BlockReference because we set a filter in 
                '  our PromptEntityOptions
                Dim blkRef As BlockReference = _
                trans.GetObject(res.ObjectId, OpenMode.ForRead)
                ' Record the ObjectIds of all AttributeReferences 
                '  attached to the BlockReference.
                Dim attRefColl As AttributeCollection = _
                blkRef.AttributeCollection()
                ReDim objIds(attRefColl.Count)
                attRefColl.CopyTo(objIds, 0)
                ReDim Preserve objIds(attRefColl.Count - 1)
            End Using

            'We only want to create our overrule instance once, 
            ' so we check if it already exists before we create it
            ' (i.e. this may be the 2nd time we've run the command)
            If myOverrule Is Nothing Then
                'Instantiate our overrule class
                myOverrule = New KeepStraightOverrule
                'Register the overrule
                Overrule.AddOverrule( _
                  RXClass.GetClass(GetType(AttributeReference)), _
                  myOverrule, False)
            End If

            'Specify which Attributes will be overruled
            myOverrule.SetIdFilter(objIds)

            'Make sure overruling is turned on so our overrule works
            Overrule.Overruling = True

        End Sub
    End Class

    'Our custom overrule class derived from TransformOverrule
    Public Class KeepStraightOverrule
        Inherits TransformOverrule

        'We want to change how an AttributeReference responds to being
        ' transformed (moved, rotated, etc.), so we override its
        ' standard TransformBy function.
        Public Overrides Sub TransformBy(ByVal entity As Entity, _
                                         ByVal transform As Matrix3d)

            'Call the normal TransformBy function for the attribute
            ' reference we're overruling.
            MyBase.TransformBy(entity, transform)
            'We know entity must be an AttributeReference because 
            ' that is the only entity we registered the overrule for.
            Dim attRef As AttributeReference = entity
            'Set rotation of attribute reference to 0 (horizontal)
            attRef.Rotation = 0.0

        End Sub
    End Class
End Namespace
